---
tags: todo, good-first-task
---
# Contribution Guide

Foam is open to contributions of any kind, including but not limited to code, documentation, ideas, and feedback.
This guide aims to help guide new and seasoned contributors getting around the Foam codebase.

## Getting Up To Speed

Before you start contributing we recommend that you read the following links:

- [[principles]] - This document describes the guiding principles behind Foam.
- [[code-of-conduct]] - Rules we hope every contributor aims to follow, allowing everyone to participate in our community!

## Diving In

We understand that diving in an unfamiliar codebase may seem scary,
to make it easier for new contributors we provide some resources:

You can also see [existing issues](https://github.com/foambubble/foam/issues) and help out!
Finally, the easiest way to help, is to use it and provide feedback by [submitting issues](https://github.com/foambubble/foam/issues/new/choose) or participating in the [Foam Community Discord](https://foambubble.github.io/join-discord/g)!

## Contributing

If you're interested in contributing, this short guide will help you get things set up locally.

1. Clone the repo locally:

   `git clone https://github.com/foambubble/foam.git`

2. Install the necessary dependencies by running this command from the root:

   `yarn install`

3. From the root, run the command:

   `yarn build`

You should now be ready to start working!

### Structure of the project

Foam code and documentation live in the monorepo at [foambubble/foam](https://github.com/foambubble/foam/).

- [/docs](https://github.com/foambubble/foam/tree/master/docs): documentation and [[recipes]].

Exceptions to the monorepo are:

- The starter template at [foambubble/foam-template](https://github.com/foambubble/)
- All other [[recommended-extensions]] live in their respective GitHub repos

This project uses [Yarn workspaces](https://classic.yarnpkg.com/en/docs/workspaces/). 

Originally Foam had:

- [/packages/foam-core](https://github.com/foambubble/foam/tree/master/packages/foam-core) - Powers the core functionality in Foam across all platforms.
- [/packages/foam-vscode](https://github.com/foambubble/foam/tree/master/packages/foam-vscode) - The core VS Code plugin.

To improve DX we have moved the `foam-core` module into `packages/foam-vscode/src/core`, but from a development point of view it's useful to think of the `foam-vscode/src/core` "submodule" as something that might be extracted in the future.

For all intents and purposes this means two things:

1. nothing in `foam-vscode/src/core` should depend on files outside of this directory
2. code in `foam-vscode/src/core` should NOT depend on `vscode` library

We have kept the yarn workspace for the time being as we might use it to pull out `foam-core` in the future, or we might need it for other packages that the VS Code plugin could depend upon (e.g. currently the graph visualization is inside the module, but it might be pulled out if its complexity increases).

### Testing

Code needs to come with tests.
We use the following convention in Foam:

- `*.test.ts` are unit tests
- `*.spec.ts` are integration tests

Tests live alongside the code in `src`.

### The VS Code Extension

This guide assumes you read the previous instructions and you're set up to work on Foam.

1. Now we'll use the launch configuration defined at [`.vscode/launch.json`](https://github.com/foambubble/foam/blob/master/.vscode/launch.json) to start a new extension host of VS Code. Open the "Run and Debug" Activity (the icon with the bug on the far left) and select "Run VSCode Extension" in the pop-up menu. Now hit F5 or click the green arrow "play" button to fire up a new copy of VS Code with your extension installed.

2. In the new extension host of VS Code that launched, open a Foam workspace (e.g. your personal one, or a test-specific one created from [foam-template](https://github.com/foambubble/foam-template)). This is strictly not necessary, but the extension won't auto-run unless it's in a workspace with a `.vscode/foam.json` file.

3. Test a command to make sure it's working as expected. Open the Command Palette (Ctrl/Cmd + Shift + P) and select "Foam: Update Markdown Reference List". If you see no errors, it's good to go!

---

Feel free to modify and submit a PR if this guide is out-of-date or contains errors!

---

[//begin]: # "Autogenerated link references for markdown compatibility"
[principles]: ../principles.md "Principles"
[code-of-conduct]: code-of-conduct.md "Code of Conduct"
[recipes]: ../user/recipes/recipes.md "Recipes"
[recommended-extensions]: ../user/getting-started/recommended-extensions.md "Recommended Extensions"
[//end]: # "Autogenerated link references"
